# -*- coding:utf-8 -*-

import collections
from inspur_sm_sdk.command import RestFunc
from inspur_sm_sdk.interface.CommonM6 import CommonM6
from inspur_sm_sdk.interface.ResEntity import ResultBean


class CommonM6_41401(CommonM6):

    def setpdisk(self, client, args):
        # login
        headers = RestFunc.login_M6(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(
                ["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)
        try:
            pdisk = setPhysicalDrive(client, args)
        except BaseException:
            RestFunc.logout(client)
        RestFunc.logout(client)
        return pdisk

    def setldisk(self, client, args):
        # login
        headers = RestFunc.login_M6(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(
                ["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)
        try:
            ldisk = setVirtualDrive(client, args)
        except BaseException:
            RestFunc.logout(client)
        RestFunc.logout(client)
        return ldisk


def setPhysicalDrive(client, args):
    result = ResultBean()
    ctrl_type_dict = {
        "LSI": [],
        "PMC": []
    }
    ctrl_id_name_dict = {}
    ctrl_id_list = []
    res = RestFunc.getRaidCtrlInfo(client)
    if res.get('code') == 0 and res.get('data') is not None:
        ctrls = res.get('data')
        for ctrl in ctrls:
            if str(ctrl.get("RaidType")).upper() == "PMC":
                ctrl_type_dict['PMC'].append(ctrl["Name"])
            else:
                ctrl_type_dict['LSI'].append(ctrl["Name"])
            if "Index" in ctrl.keys():
                ctrl_id_name_dict[ctrl["Index"]] = ctrl["Name"]
                ctrl_id_list.append(str(ctrl["Index"]))
            elif "id" in ctrl.keys():
                ctrl_id_name_dict[ctrl["id"]] = ctrl["Name"]
                ctrl_id_list.append(str(ctrl["id"]))
    else:
        result.State("Failure")
        result.Message(["ctrl Information Request Fail!" + res.get('data')])
        return result
    if ctrl_id_list == []:
        result.State("Failure")
        result.Message(["No raid controller!"])
        return result

    ctrl_list_dict = {}
    pds = {}
    res = RestFunc.getPhysicalDiskInfo(client)
    if res.get('code') == 0 and res.get('data') is not None:
        pds = res.get('data')
        for pd in pds:
            if pd['ControllerName'] not in ctrl_list_dict:
                ctrl_list_dict[pd['ControllerName']] = []
            ctrl_list_dict[pd['ControllerName']].append(pd['DeviceID'])
    else:
        result.State("Failure")
        result.Message([res.get('data')])

    for pd in ctrl_list_dict:
        ctrl_list_dict.get(pd).sort()

    if args.Info is not None:
        LSI_flag = False
        raidList = []
        for ctrlid in ctrl_id_name_dict:
            raidDict = collections.OrderedDict()
            raidDict['Controller ID'] = ctrlid
            raidDict['Controller Name'] = ctrl_id_name_dict.get(ctrlid)
            pdiskList = []
            for pd in pds:
                if pd.get("ControllerName") == ctrl_id_name_dict.get(ctrlid):
                    LSI_flag = True
                    pdiskDict = collections.OrderedDict()
                    pdiskDict['Physical Drive ID'] = pd.get("DeviceID")
                    pdiskDict['Physical Drive Name'] = pd.get("Name")
                    pdiskDict['Interface Type'] = pd.get("InterfaceType")
                    pdiskDict['Media Type'] = pd.get("MediaType")
                    pdiskDict['Non Coerced Size'] = pd.get("NONCoercedSize")
                    pdiskDict['Firmware State'] = pd.get("FWState")
                    pdiskList.append(pdiskDict)
            raidDict['pdisk'] = pdiskList
        raidList.append(raidDict)
        if not LSI_flag:
            result.State('Failure')
            result.Message(['No LSI raid controller'])
            return result
        result.State('Success')
        result.Message(raidList)
        return result

    if args.ctrlId is None:
        result.State('Failure')
        result.Message(['Controller id is needed.'])
        return result
    if args.deviceId is None:
        result.State('Failure')
        result.Message(['Physical drive id is needed.'])
        return result

    if args.ctrlId not in ctrl_id_name_dict:
        result.State('Failure')
        result.Message(["Invalid controller id, choose from " + ",".join(ctrl_id_list) + "."])
        return result

    the_pd_list = ctrl_list_dict.get(ctrl_id_name_dict.get(args.ctrlId))
    if str(ctrl_id_name_dict.get(args.ctrlId)) in ctrl_type_dict.get('PMC'):
        ctrl_type = "PMC"
        if args.option != "LOC" and args.option != "STL":
            result.State('Failure')
            result.Message(["Physical drive under PMC raid controller only support LOC and STL."])
            return result
        if args.option == "LOC" and args.duration is None:
            result.State('Failure')
            result.Message(["Please input duration parameter while setting LOC of physical drive under PMC raid controller"])
            return result
    else:
        ctrl_type = "LSI"
    args.ctrl_type = ctrl_type
    if args.deviceId not in the_pd_list:
        result.State('Failure')
        result.Message(["Invalid physical drive slot num, choose from " + str(the_pd_list)])
        return result

    args.location = None
    args.erase = None
    args.status = None
    if args.option == 'LOC':
        args.location = 'StartLocate'
    elif args.option == 'STL':
        args.location = 'StopLocate'
    elif args.option == 'ES':
        args.erase = 'EraseStop'
    elif args.option == 'EM':
        args.erase = 'EraseSimple'
    elif args.option == 'EN':
        args.erase = 'EraseNormal'
    elif args.option == 'ET':
        args.erase = 'EraseThrough'
    elif args.option == 'UG':
        args.status = 'UNCONFIGURED GOOD'
    elif args.option == 'UB':
        args.status = 'UNCONFIGURED BAD'
    elif args.option == 'OFF':
        args.status = 'OFFLINE'
    elif args.option == 'ON':
        args.status = 'ONLINE'
    elif args.option == 'JB':
        args.status = 'JBOD'
    if args.location is not None:
        if args.ctrl_type == "LSI":
            res = RestFunc.locateDiskByRest(client, args.ctrlId, args.deviceId, args.location)
        else:
            if args.duration is not None:
                if args.duration < 1 or args.duration > 255:
                    result.State("Failure")
                    result.Message(["Please enter an integer from 1 to 255."])
                    return result
            res = RestFunc.locateDiskByRestPMC_41401(client, args.ctrlId, args.deviceId, args.location,
                                                     args.duration)
    elif args.erase is not None:
        res = RestFunc.erasePhysicalDisk(
            client, args.ctrlId, args.deviceId, args.erase)
    elif args.status is not None:
        res = RestFunc.setPhysicalDisk(
            client, args.ctrlId, args.deviceId, args.status)
    if res == {}:
        result.State("Failure")
        result.Message(["disk operation failed"])
    elif res.get('code') == 0 and res.get('data') is not None:
        result.State('Success')
        result.Message(
            ['operation is successful,please wait a few seconds.'])
    else:
        result.State("Failure")
        result.Message(
            ['operation failed, ' + str(res.get('data'))])
    return result


def setVirtualDrive(client, args):
    result = ResultBean()
    ctrl_id_name_dict = {}
    ctrl_type_dict = {
        "LSI": [],
        "PMC": []
    }
    ctrl_id_list = []
    res = RestFunc.getRaidCtrlInfo(client)
    if res.get('code') == 0 and res.get('data') is not None:
        ctrls = res.get('data')
        for ctrl in ctrls:
            if str(ctrl.get("RaidType")).upper() == "PMC":
                ctrl_type_dict['PMC'].append(ctrl["Name"])
            else:
                ctrl_type_dict['LSI'].append(ctrl["Name"])
            if "Index" in ctrl.keys():
                ctrl_id_name_dict[ctrl["Index"]] = ctrl["Name"]
                ctrl_id_list.append(str(ctrl["Index"]))
            elif "id" in ctrl.keys():
                ctrl_id_name_dict[ctrl["id"]] = ctrl["Name"]
                ctrl_id_list.append(str(ctrl["id"]))
    else:
        result.State("Failure")
        result.Message(["ctrl Information Request Fail!" + res.get('data')])
        return result
    if ctrl_id_list == []:
        result.State("Failure")
        result.Message(["No raid controller!"])
        return result
    # ld
    ctrl_ld_list_dict = {}
    lds = {}
    res = RestFunc.getLogicalDiskInfo(client)
    if res.get('code') == 0 and res.get('data') is not None:
        lds = res.get('data')
        for ld in lds:
            if ld['ControllerName'] not in ctrl_ld_list_dict:
                ctrl_ld_list_dict[ld['ControllerName']] = []
            if "Index" in ld:
                ctrl_ld_list_dict[ld['ControllerName']].append(ld['Index'])
            elif "TargetID" in ld:
                ctrl_ld_list_dict[ld['ControllerName']].append(ld['TargetID'])
    else:
        result.State("Failure")
        result.Message([res.get('data')])
        return result

    for pd in ctrl_ld_list_dict:
        ctrl_ld_list_dict.get(pd).sort()

    if args.Info is not None:
        LSI_flag = False
        raidList = []
        for ctrlid in ctrl_id_name_dict:
            raidDict = collections.OrderedDict()
            raidDict['Controller ID'] = ctrlid
            raidDict['Controller Name'] = ctrl_id_name_dict.get(ctrlid)
            raidDict['Virtual Drive ID'] = ctrl_ld_list_dict.get(ctrl_id_name_dict.get(ctrlid))
            ldiskList = []
            for ld in lds:
                if ld.get("ControllerName") == ctrl_id_name_dict.get(ctrlid):
                    LSI_flag = True
                    ldiskDict = collections.OrderedDict()
                    ldiskDict['Virtual Drive ID'] = ld.get("Index")
                    ldiskDict['Capacity (GB)'] = ld.get("CAPACITY")
                    ldiskDict['Raid Level'] = ld.get("VOLUME_RAID_LEVEL")
                    ldiskDict['PhysicalDisks'] = ld.get("PhysicalDisks")
                    ldiskDict['Firmware State'] = ld.get("State")
                    ldiskList.append(ldiskDict)
            raidDict['ldisk'] = ldiskList
            raidList.append(raidDict)
        if not LSI_flag:
            result.State('Failure')
            result.Message(['No LSI raid controller'])
            return result
        result.State('Success')
        result.Message(raidList)
        return result

    if args.ctrlId is None:
        result.State('Failure')
        result.Message(['Controller id is needed.'])
        return result
    if args.ldiskId is None:
        result.State('Failure')
        result.Message(['Logical drive id is needed.'])
        return result

    if args.ctrlId not in ctrl_id_name_dict:
        result.State('Failure')
        result.Message(["Invalid controller id, choose from " + ",".join(ctrl_id_list) + "."])
        return result

    the_ld_list = ctrl_ld_list_dict.get(ctrl_id_name_dict.get(args.ctrlId))
    if str(ctrl_id_name_dict.get(args.ctrlId)) in ctrl_type_dict.get('PMC'):
        ctrl_type = "PMC"
        if args.option != "LOC" and args.option != "STL" and args.option != "DEL":
            result.State('Failure')
            result.Message(["Logical drive under PMC raid controller only support LOC, STL, DEL."])
            return result
        if args.option == "LOC" and args.duration is None:
            result.State('Failure')
            result.Message(["Please input duration parameter while setting LOC of logical drive under PMC raid controller."])
            return result
    else:
        ctrl_type = "LSI"

    args.ctrl_type = ctrl_type
    args.location = None
    args.init = None
    args.delete = None
    if args.ldiskId not in the_ld_list:
        result.State('Failure')
        result.Message(["Invalid virtual drive id, choose from " + str(the_ld_list)])
        return result
    if args.option == 'LOC':
        args.location = 'StartLocate'
    elif args.option == 'STL':
        args.location = 'StopLocate'
    elif args.option == 'FI':
        args.init = 'FastInit'
    elif args.option == 'SFI':
        args.init = 'SlowFullInit'
    elif args.option == 'SI':
        args.init = 'StopInit'
    elif args.option == 'DEL':
        args.delete = "NotNone"
    if args.location is not None:
        if args.ctrl_type == "LSI":
            res = RestFunc.locateLogicalDisk(client, args.ctrlId, args.ldiskId, args.location)
        else:
            if args.duration is not None:
                if args.duration < 1 or args.duration > 255:
                    result.State("Failure")
                    result.Message(["Please enter an integer from 1 to 255."])
                    return result
            res = RestFunc.locateLogicalDiskPMC(client, args.ctrlId, args.ldiskId, args.location, args.duration)
    elif args.init is not None:
        res = RestFunc.initLogicalDisk(client, args.ctrlId, args.ldiskId, args.init)
    elif args.delete is not None:
        if args.ctrl_type == "LSI":
            res = RestFunc.deleteLogicalDisk(client, args.ctrlId, args.ldiskId)
        else:
            res = RestFunc.deleteLogicalDiskPMC(client, args.ctrlId, args.ldiskId)
    if res == {}:
        result.State("Failure")
        result.Message(["disk operation failed"])
    elif res.get('code') == 0 and res.get('data') is not None:
        result.State('Success')
        result.Message(
            ['operation is successful,please wait a few seconds.'])
    else:
        result.State("Failure")
        result.Message(
            ['operation failed, ' + str(res.get('data'))])
    return result


def addPMCLogicalDisk(client, args, pds, ctrl_id_name_dict):
    result = ResultBean()
    if args.size is None or args.vname is None or args.accelerator is None or args.slot is None or \
            args.rlevel is None:
        result.State('Failure')
        result.Message(['some parameters are missing'])
        return result

    # args.pd
    args.pdlist = args.slot
    pd_para_len = len(args.pdlist)

    # set raid
    if args.rlevel == 1:
        if pd_para_len < 2:
            result.State('Failure')
            result.Message(['raid 1 need 2 disks at least'])
            return result
    elif args.rlevel == 5:
        if pd_para_len < 3:
            result.State('Failure')
            result.Message(['raid 5 need 3 disks at least'])
            return result
    elif args.rlevel == 6:
        if pd_para_len < 4:
            result.State('Failure')
            result.Message(['raid 6 need 4 disks at least'])
            return result
    elif args.rlevel == 10:
        if pd_para_len < 4:
            result.State('Failure')
            result.Message(['raid 10 need 4 disks at least'])
            return result
    raid_dict = {0: "raid0", 1: "raid1", 5: "raid5", 6: "raid6", 10: "raid10"}
    stripsize_dict_pmc = {1: 7, 2: 8, 3: 9, 4: 10, 5: 11}
    args.rlevel = raid_dict.get(args.rlevel)
    args.size = stripsize_dict_pmc.get(args.size)
    pd_dev_list = []
    for pd_slot_num in args.pdlist:
        for pd in pds:
            if pd['ControllerName'] == ctrl_id_name_dict.get(args.ctrlId) and pd['SlotNum'] == int(pd_slot_num):
                pd_dev_list.append(pd['DeviceID'])
                if pd.get("array_number") != 65535:
                    result.State('Failure')
                    result.Message(['The array number of physical disk is ' + pd.get("array_number")
                                    + ", logical disk can be created only when its array number is 65535."])
                    return result
    deviceId = "~".join(pd_dev_list)
    data = {
        "CTRLID": args.ctrlId,
        "StripeSize": args.size,
        "VDNAME": args.vname,
        "accelerator": args.accelerator,
        "array_number": 65535,
        "deviceId": deviceId,
        "numdrives": len(pd_dev_list),
        "parity_group_count": 1,
        "raid_level": args.rlevel,
        "vendor_type": 1
    }
    res = RestFunc.createVirtualDrive_41401(client, data)
    if res == {}:
        result.State("Failure")
        result.Message(["create virtual drive failed"])
    elif res.get('code') == 0 and res.get('data') is not None:
        result.State('Success')
        result.Message(['create virtual drive successful.'])
    else:
        result.State("Failure")
        result.Message(['create virtual drive failed, ' + str(res.get('data'))])
    return result



# -*- coding:utf-8 -*-

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class IBase():
    def __init__(self):
        pass

    def getfru(self, client, args):
        """
        get product Fru information
        :return:
        """

    def getProdcut(self, client, args):
        """

        :return:
        """

    def getcapabilities(self, client, args):
        """

        :return:
        """
        print('cap')

    def getcpu(self, client, args):
        """

        :return:
        """

    def getmemory(self, client, args):
        """

        :return:
        """

    def powercontrol(self, client, args):
        """

        :return:
        """

    def gethealth(self, client, args):
        """

        :return:
        """

    def getsysboot(self, client, args):
        """

        :return:
        """

    def geteventlog(self, client, args):
        """

        :return:
        """

    def downloadtfalog(self, client, args):
        """

        :return:
        """

    def gethealthevent(self, client, args):
        """

        :return:
        """

    def getraid(self, client, args):
        """

        :return:
        """

    def getldisk(self, client, args):
        """

        :return:
        """

    def clearsel(self, client, args):
        """

        :return:
        """

    def getnic(self, client, args):
        """

        :return:
        """

    def setsysboot(self, client, args):
        """

        :return:
        """

    def powerctrldisk(self, client, args):
        """

        :return:
        """

    def powerctrlpcie(self, client, args):
        """

        :return:
        """

    def getbios(self, client, args):
        """

        :return:
        """

    def setbios(self, client, args):
        """

        :return:
        """

    def setbiospwd(self, client, args):
        """

        :return:
        """

    def getbiossetting(self, client, args):
        """

        :return:
        """

    def getbiosresult(self, client, args):
        """

        :return:
        """

    def getbiosdebug(self, client, args):
        """

        :return:
        """

    def clearbiospwd(self, client, args):
        """

        :return:
        """

    def restorebios(self, client, args):
        """

        :return:
        """

    def setbiosdebug(self, client, args):
        """

        :return:
        """

    def mountvmm(self, client, args):
        """

        :return:
        """

    def setthreshold(self, client, args):
        """

        :return:
        """

    def downloadfandiag(self, client, args):
        """

        :return:
        """

    def downloadsol(self, client, args):
        """

        :return:
        """

    def sethsc(self, client, args):
        """

        :return:
        """

    def chassis(self, client, args):
        """

        :return:
        """

    def getproduct(self, client, args):
        """

        :return:
        """

    def setproductserial(self, client, args):
        """

        :return:
        """

    def getfan(self, client, args):
        """

        :return:
        """

    def fancontrol(self, client, args):
        """

        :return:
        """

    def getsensor(self, client, args):
        """

        :return:
        """

    def gettemp(self, client, args):
        """

        :return:
        """

    def getvolt(self, client, args):
        """

        :return:
        """

    def getthreshold(self, client, args):
        """

        :return:
        """

    def getpwrcap(self, client, args):
        """

        :return:
        """

    def getpsu(self, client, args):
        """

        :return:
        """

    def recoverypsu(self, client, args):
        """

        :return:
        """

    def getpdisk(self, client, args):
        """

        :return:
        """

    def getpcie(self, client, args):
        """

        :return:
        """

    def locateserver(self, client, args):
        """

        :return:
        """

    def locatedisk(self, client, args):
        """

        :return:
        """

    def Managers(self, client, args):
        """

        :return:
        """

    def getip(self, client, args):
        """

        :return:
        """

    def getdns(self, client, args):
        """

        :return:
        """

    def gettrap(self, client, args):
        """

        :return:
        """

    def restorebmc(self, client, args):
        """

        :return:
        """

    def collect(self, client, args):
        """

        :return:
        """

    def settime(self, client, args):
        """

        :return:
        """

    def settimezone(self, client, args):
        """

        :return:
        """

    def resetbmc(self, client, args):
        """

        :return:
        """

    def setip(self, client, args):
        """

        :return:
        """

    def setvlan(self, client, args):
        """

        :return:
        """

    def getvnc(self, client, args):
        """

        :return:
        """

    def setvnc(self, client, args):
        """

        :return:
        """

    def getservice(self, client, args):
        """

        :return:
        """

    def setservice(self, client, args):
        """

        :return:
        """

    def getmgmtport(self, client, args):
        """

        :return:
        """

    def getserialport(self, client, args):
        """

        :return:
        """

    def setserialport(self, client, args):
        """

        :return:
        """

    def setadaptiveport(self, client, args):
        """

        :return:
        """

    def settrapcom(self, client, args):
        """

        :return:
        """

    def settrapdest(self, client, args):
        """

        :return:
        """

    def triggernmi(self, client, args):
        """

        :return:
        """

    def exportbmccfg(self, client, args):
        """

        :return:
        """

    def importbmccfg(self, client, args):
        """

        :return:
        """

    def exportbioscfg(self, client, args):
        """

        :return:
        """

    def importbioscfg(self, client, args):
        """

        :return:
        """

    def delvncsession(self, client, args):
        """

        :return:
        """

    def sendipmirawcmd(self, client, args):
        """

        :return:
        """

    def AccountService(self, client, args):
        """

        :return:
        """

    def getuser(self, client, args):
        """

        :return:
        """

    def adduser(self, client, args):
        """

        :return:
        """

    def deluser(self, client, args):
        """

        :return:
        """

    def setpwd(self, client, args):
        """

        :return:
        """

    def setpriv(self, client, args):
        """

        :return:
        """

    def updateservice(self, client, args):
        """

        :return:
        """

    def getfw(self, client, args):
        """

        :return:
        """

    def getupdatestate(self, client, args):
        """

        :return:
        """

    def fwupdate(self, client, args):
        """

        :return:
        """

    def EventServices(self, client, args):
        """

        :return:
        """

    def geteventsub(self, client, args):
        """

        :return:
        """

    def addeventsub(self, client, args):
        """

        :return:
        """

    def deleventsub(self, client, args):
        """

        :return:
        """

    def TaskService(self, client, args):
        """

        :return:
        """

    def gettaskstate(self, client, args):
        """

        :return:
        """

    def cancletask(self, client, args):
        """

        :return:
        """



# -*- coding:utf-8 -*-
import re
import os


def parseUrl(fileurl):
    if fileurl == ".":
        file_name = ""
        file_path = os.path.abspath(".")
    elif fileurl == "..":
        file_name = ""
        file_path = os.path.abspath("..")
    elif os.path.isdir(fileurl):
        file_name = ""
        file_path = os.path.abspath(fileurl)
    else:
        file_name = os.path.basename(fileurl)
        file_path = os.path.dirname(fileurl)

    # 只输入文件名字，则默认为当前路径

    if file_path == "":
        file_path = os.path.abspath(".")
    return True, file_path, file_name


def checkUrl(fileurl):
    file_name = os.path.basename(fileurl)
    file_path = os.path.dirname(fileurl)

    #fileurl = os.path.join(file_path, file_name)

    if not os.path.exists(file_path):
        try:
            os.makedirs(file_path)
        except BaseException:
            return False, "can not create path."
    else:
        if os.path.exists(fileurl):
            name_id = 1
            name_extension = os.path.splitext(file_name)[1]
            name_prefix = os.path.splitext(file_name)[0]
            if name_extension == ".gz" and os.path.splitext(name_prefix)[
                    1] == ".tar":
                name_prefix = os.path.splitext(name_prefix)[0]
                name_extension = ".tar.gz"
            name_new = name_prefix + "(1)" + name_extension
            file_new = os.path.join(file_path, name_new)
            while os.path.exists(file_new):
                name_id = name_id + 1
                name_new = name_prefix + \
                    "(" + str(name_id) + ")" + name_extension
                file_new = os.path.join(file_path, name_new)
            fileurl = file_new

    return True, fileurl



# -*- coding:utf-8 -*-
'''
#=========================================================================
#   @Description: RedfishFunc Class
#
#   @author: zhong
#   @Date:
#=========================================================================
'''

from requests.auth import HTTPBasicAuth


def exportbiosoption(client, header):
    JSON = {}
    r = client.request("POST", "redfish/v1/Systems/1/Actions/BIOS.ExportConfiguration",
                       headers=header)
    if r is None:
        JSON["code"] = 1
        JSON["data"] = 'Failed to call BMC interface redfish/v1/Systems/system_id/Actions/BIOS.ExportConfiguration, ' \
                       'response is none'
    elif r.status_code == 200:
        try:
            result = r.json()
            JSON["code"] = 0
            JSON["data"] = result
        except Exception as e:
            JSON["code"] = 1
            JSON["data"] = e.message
    else:
        try:
            result = r.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(
                result["error"]["message"]) + ' the status code is ' + str(r.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(r.status_code)
    return JSON


def importbiosoption(client, header, file):
    JSON = {}
    try:
        f = open(file, 'rb')
        files = [("config", f)]
        r = client.request("POST", "redfish/v1/Systems/1/Actions/BIOS.ImportConfiguration", headers=header, files=files,
                           data=None)
        f.close()
        if r is None:
            JSON["code"] = 1
            JSON["data"] = 'Failed to call BMC interface redfish/v1/Systems/1/Actions/BIOS.ImportConfiguration , ' \
                           'response is none'
        elif r.status_code == 200:
            JSON["code"] = 0
            JSON["data"] = "Import BIOS option files completed."
        else:
            try:
                result = r.json()
                JSON['code'] = 2
                JSON['data'] = 'request failed, response content: ' + str(
                    result["error"]["message"]) + ' the status code is ' + str(r.status_code) + "."
            except:
                JSON['code'] = 1
                JSON['data'] = 'request failed, response status code is ' + str(r.status_code)
    except Exception as e:
        JSON['code'] = 1
        JSON['data'] = str(e)
    return JSON


def getBiosByRedfish(client):
    JSON = {}
    response = client.request("GET", "redfish/v1/Systems/Self/Bios",
                              auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        try:
            result = response.json()
            JSON['code'] = 0
            JSON['data'] = result.get('Attributes')
        except Exception as e:
            JSON['code'] = 1
            JSON['data'] = e.message
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(
                res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def getBiosV1ByRedfish(client, login_header):
    JSON = {}
    response = client.request("GET", "redfish/v1/Systems/1/Bios", headers=login_header)
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        try:
            result = response.json()
            JSON['code'] = 0
            JSON['data'] = result.get('Attributes')
            JSON['headers'] = response.headers
        except Exception as e:
            JSON['code'] = 1
            JSON['data'] = e.message
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(
                res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def getBiosSDByRedfish(client):
    JSON = {}
    response = client.request("GET", "redfish/v1/Systems/Self/Bios/SD", auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        result = response.json()
        # print(result)
        JSON['code'] = 0
        JSON['data'] = result.get('Attributes')
    elif response.status_code == 404:
        JSON['code'] = 0
        JSON['data'] = {}
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def setBiosSDByRedfish(client, data):
    JSON = {}
    header = {"Content-Type": "application/json", "If-None-Match": "none"}
    response = client.request("PATCH", "redfish/v1/Systems/Self/Bios/SD", json=data, headers=header, auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 204 or response.status_code == 200:
        JSON['code'] = 0
        JSON['data'] = ""
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def setBiosV1SDByRedfish(client, data, headers, login_header):
    JSON = {}
    # header = {"Content-Type":"application/json", "If-None-Match":"none"}
    header = {"If-Match": headers['etag'],
              "X-Auth-Token": login_header['X-Auth-Token']
              }
    response = client.request("PATCH", "redfish/v1/Systems/1/Bios/Settings", json=data, headers=header)
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 204 or response.status_code == 200:
        JSON['code'] = 0
        JSON['data'] = ""
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def getBiosSRByRedfish(client):
    JSON = {}
    response = client.request("GET", "redfish/v1/Systems/Self/Bios/SR", auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        result = response.json()
        # print(result)
        JSON['code'] = 0
        JSON['data'] = result.get('SettingResult')
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def setBiosPwdByRedfish(client, data):
    JSON = {}
    response = client.request("POST", "redfish/v1/Systems/Self/Bios/Actions/Bios.ChangePassword", json=data,
                              auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 204 or response.status_code == 200:
        JSON['code'] = 0
        JSON['data'] = ""
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def setBiosPwdM6(client, data):
    JSON = {}
    header = {"Content-Type": "application/json", "If-None-Match": "none"}
    response = client.request("PATCH", "redfish/v1/AccountService/Accounts/1", json=data, headers=header, auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 204 or response.status_code == 200:
        JSON['code'] = 0
        JSON['data'] = ""
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def clearBiosPwdByRedfish(client, data):
    JSON = {}
    response = client.request("POST", "redfish/v1/Systems/Self/Bios/Actions/Bios.ChangePassword", json=data,
                              auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 204:
        JSON['code'] = 0
        JSON['data'] = ""
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def resetBiosByRedfish(client, data):
    JSON = {}
    response = client.request("POST", "redfish/v1/Systems/Self/Bios/Actions/Bios.ResetBios", json=data, auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 204 or response.status_code == 200:
        JSON['code'] = 0
        JSON['data'] = ""
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
            # data = res["error"]['@Message.ExtendedInfo']
            # ldata = len(data)
            # message = ''
            # for i in range(ldata):
            #     message += str(data[i].get('Message'))+' '
            # JSON['data']='request failed, response content: ' + str(message)+ 'the status code is ' + str(response.status_code)+"."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def getChassisID(client):
    JSON = {}
    response = client.request("GET", "redfish/v1/Chassis",
                              auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        try:
            result = response.json()
            JSON['code'] = 0
            JSON['data'] = result.get('Members')
        except Exception as e:
            JSON['code'] = 1
            JSON['data'] = e
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(
                res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def getPCIEDevices(client, chassis_id):
    JSON = {}
    response = client.request("GET", "%s/PCIeDevices" % str(chassis_id)[1:],
                              auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        try:
            result = response.json()
            JSON['code'] = 0
            JSON['data'] = result.get('Members')
        except Exception as e:
            JSON['code'] = 1
            JSON['data'] = e
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(
                res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def getPCIEInfo(client, url):
    JSON = {}
    response = client.request("GET", url, auth=HTTPBasicAuth(client.username, client.passcode))
    if response is None:
        JSON['code'] = 1
        JSON['data'] = 'response is none'
    elif response.status_code == 200:
        try:
            result = response.json()
            JSON['code'] = 0
            JSON['data'] = result
        except Exception as e:
            JSON['code'] = 1
            JSON['data'] = e
    else:
        try:
            res = response.json()
            JSON['code'] = 2
            JSON['data'] = 'request failed, response content: ' + str(
                res["error"]["message"]) + ' the status code is ' + str(response.status_code) + "."
        except:
            JSON['code'] = 1
            JSON['data'] = 'request failed, response status code is ' + str(response.status_code)
    return JSON


def login(client):
    data = {
        "UserName": str(client.username),
        "Password": str(client.passcode),
        "SessionTimeOut": "600"
    }
    headers = {'Content-Type': 'application/json'}
    header = {}
    # print('responds')
    responds = client.request('POST', 'redfish/v1/SessionService/Sessions', headers=headers, json=data, data=None)
    login_id = ''
    try:
        if responds is not None:
            if responds.status_code == 201:  # 登录成功
                XCSRFToken = responds.headers['X-Auth-Token']
                header = {
                    "X-Auth-Token": XCSRFToken,
                }
                login_id = responds.json()['Id']
    except Exception as e:
        login_id = "login error, " + str(e)
    return header, login_id


def logout(client, login_id, login_header):
    token = login_header.get('X-Auth-Token')
    headers = {"X-Auth-Token": token}
    # headers = {'Content-Type': 'application/json'}
    responds = client.request("DELETE", "redfish/v1/SessionService/Sessions/" + str(login_id), headers=headers)
    # print(responds)



# -*- coding:utf-8 -*-

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from inspur_sm_sdk.interface.NF5280M5_435 import NF5280M5_435
from inspur_sm_sdk.command import RestFunc
from inspur_sm_sdk.interface.ResEntity import ResultBean
import os


class NF5280M5_436(NF5280M5_435):
    def exportbioscfg(self, client, args):
        """
        export bios setup configuration
        :param client:
        :param args:
        :return:
        """
        export = ResultBean()

        file_path = os.path.dirname(args.fileurl)
        file_name = os.path.basename(args.fileurl)

        if not os.path.exists(file_path):
            try:
                os.makedirs(file_path)
            except BaseException:
                export.State("Failure")
                export.Message(["cannot build path."])
                return export
        if '.json' not in file_name and '.conf' not in file_name:
            export.State("Failure")
            export.Message(["please input filename with suffix .json/.conf."])
            return export
        # login
        headers = RestFunc.login(client)
        if headers == {}:
            login_res = ResultBean()
            login_res.State("Failure")
            login_res.Message(
                ["login error, please check username/password/host/port"])
            return login_res
        client.setHearder(headers)
        # get
        res = RestFunc.exportTwoBios_436_CfgByRest(
            client, args.fileurl, file_name)

        if res == {}:
            export.State("Failure")
            export.Message(["export bios setup configuration file failed."])
        elif res.get('code') == 0:
            export.State('Success')
            export.Message([res.get('data')])
        elif res.get('code') == 4:
            export.State('Failure')
            export.Message([res.get('data')])
        else:
            export.State("Failure")
            export.Message(
                ["export bios setup configuration file error, " + res.get('data')])
        # logout
        RestFunc.logout(client)
        return export



# -*- coding:utf-8 -*-

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import copy
import os
import yaml
import sys
import xml.etree.ElementTree as ET

routePath = os.path.join(
    os.path.abspath(
        os.path.dirname(
            os.path.dirname(__file__))),
    "route",
    "route.yml")


# get yaml config util,singleton type
class configUtil():
    def __init__(self):
        pass

    # sinlge
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, "_instance"):
            if sys.version_info < (3, 0):
                cls._instance = super(configUtil, cls).__new__(cls, *args, **kwargs)
            else:
                cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    # get all configuration option
    def getRouteConfig(self):
        '''
        get route configuration to dict
        :return: all route  dict
        '''
        if os.path.exists(routePath):
            with open(routePath, 'r') as f:
                serverconfig = yaml.load(f.read())
                return serverconfig

    def getRouteOption(self, productName, bmcVersion):
        # try:
        yaml1 = open(routePath)
        content = yaml.load(yaml1, Loader=yaml.BaseLoader)
        yaml1.close()
        if "G220-A" in productName.upper():
            productName = "G220-A"
        elif "G226-A" in productName.upper():
            productName = "G226-A"
        elif "S520-A" in productName.upper():
            productName = "S520-A"
        if content.get(productName.upper(), None):
            model_info = content.get(productName.upper())
            model_keys = copy.deepcopy(list(model_info.keys()))
            model_keys.remove('common')
            if (bmcVersion is None or len(model_keys) == 0) and model_info.get('common'):
                return model_info.get("common")
            elif len(model_keys) >= 1:
                model_keys.sort()
                # model_keys = map(eval, model_keys)
                if float(bmcVersion) < float(model_keys[0]):
                    return model_info.get("common")
                if len(model_keys) > 1:
                    for i in range(len(model_keys) - 1):
                        if float(bmcVersion) >= float(model_keys[i]) and float(bmcVersion) < float(model_keys[i + 1]):
                            return model_info.get(str(model_keys[i]))
                return model_info.get(str(model_keys[len(model_keys) - 1]))
            else:
                return "Error: Not find interface of {0}".format(productName)
        return "Error: sdk does not support {0} at present.".format(productName)


    # xmlfilepath 文件路径
    def getSetOption(self, xmlfilepath):

        tree = ET.parse(xmlfilepath)  # 调用parse方法返回解析树
        server = tree.getroot()  # 获取根节点

        blongtoSet = set()  # 存储所有的分类
        descriptionList = []  # 存储所有的描述
        infoList = []  # 存储所有xml表示的数据 字典列表

        for cfgItems in server:
            for cfgItem in cfgItems:
                info = {}  # 字典
                # name标签
                blongto_text = ''  # 此处只允许有个Item有一个描述和分类
                description_text = ''  # 只有一个描述,
                for name in cfgItem.getiterator('name'):
                    for belongto in name.getiterator('belongto'):
                        blongto_text = belongto.text
                    for description in name.getiterator('description'):
                        description_text = str(description.text).replace(" ", "")

                blongtoSet.add(blongto_text)
                descriptionList.append(description_text)

                # getter标签
                getterCMD = ''
                for getter in cfgItem.getiterator('getter'):
                    getterCMD = str(getter.text).replace('raw', '')

                # #setters标签
                setterlist = []  # 后面嵌套了setOption，是一个字典列表，每一项都是一个字典，字典里面包含{cmd，value，validation}
                sin = False
                for setters in cfgItem.getiterator('setters'):
                    for setter in setters.getiterator('setter'):
                        setOption = {}
                        for cmd in setter.getiterator('cmd'):
                            # 将tab键替换，换行键替换
                            setOption['cmd'] = cmd.text.replace("\\t", "").replace("\\n", " xxxxx ").replace('raw', '')
                        for value in setter.getiterator('value'):
                            setOption['value'] = value.text
                        for validation in setter.getiterator('validation'):
                            setOption['validation'] = validation.text
                            sin = True
                        setterlist.append(setOption)

                info['getter'] = getterCMD
                info['key'] = blongto_text + "_" + description_text
                info['description'] = description_text
                info['blongto'] = blongto_text
                info['input'] = sin
                info['setter'] = setterlist
                infoList.append(info)

        return blongtoSet, descriptionList, infoList
